// $Id: config.cpp,v 1.7 2001/07/15 12:14:54 livne Exp $
// Melchior FRANZ  <a8603365@unet.univie.ac.at>	-- 2001-04-22

#include "config.h"
#include <kconfig.h>
#include <klocale.h>
#include <kglobal.h>
#include <qwhatsthis.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : QNixConfig
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// 'conf' 	is a pointer to the kwindecoration modules open kwin config,
//			and is by default set to the "Style" group.
//
// 'parent'	is the parent of the QObject, which is a VBox inside the
//			Configure tab in kwindecoration

QNiXConfig::QNiXConfig(KConfig* conf, QWidget* parent) : QObject(parent)
{
	clientrc = new KConfig("kwinqnixrc");
	KGlobal::locale()->insertCatalogue("libkwinqnix_config");
	gb = new QGroupBox(1, Qt::Horizontal, i18n("Decoration Settings"), parent);

	cbShowMenuButtonIcon = new QCheckBox(i18n("&Menu button always shows application mini icon"), gb);
	QWhatsThis::add( cbShowMenuButtonIcon,
				i18n("When selected, all titlebar menu buttons will have "
				"the application icon shown. If not selected, the current "
				"theme's defaults are used instead.") );

	connect(cbShowMenuButtonIcon, SIGNAL(clicked()), this, SLOT(slotSelectionChanged()));

	cbShowLeftDetail = new QCheckBox(i18n("&Show left detail"), gb);
	QWhatsThis::add( cbShowLeftDetail,
				i18n("Show extra design at top left hand corner of window") );

	connect(cbShowLeftDetail, SIGNAL(clicked()), this, SLOT(slotSelectionChanged()));

	load(conf);
	gb->show();
}


QNiXConfig::~QNiXConfig()
{
	delete cbShowMenuButtonIcon;
	delete cbShowLeftDetail;
	delete gb;
	delete clientrc;
}


void QNiXConfig::slotSelectionChanged()
{
	emit changed();
}


void QNiXConfig::load(KConfig* /*conf*/)
{
	bool override ;

	clientrc->setGroup("General");

	override = clientrc->readBoolEntry( "ShowMenuButtonIcon", false );
	cbShowMenuButtonIcon->setChecked( override );

	override = clientrc->readBoolEntry( "ShowLeftDetail", true );
	cbShowLeftDetail->setChecked( override );

}


void QNiXConfig::save(KConfig* /*conf*/)
{
	clientrc->setGroup("General");
	clientrc->writeEntry("ShowMenuButtonIcon", cbShowMenuButtonIcon->isChecked());
	clientrc->writeEntry("ShowLeftDetail", cbShowLeftDetail->isChecked());
	clientrc->sync();
}


void QNiXConfig::defaults()
{
	cbShowMenuButtonIcon->setChecked( false );
	cbShowLeftDetail->setChecked( true );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern "C"
{
	QObject* allocate_config( KConfig* conf, QWidget* parent )
	{
		return(new QNiXConfig(conf, parent));
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.moc"
