/*
 * QNix      (C) 2001 Derek Greene     <del@chek.com>
 * Based on code from ModSystem & Redmond kwin clients
 *  			   (C) 2000 Daniel M. Duley  <mosfet@kde.org>
 * 					 (C) 2001 Karol Szwed <gallium@kde.org>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <qlayout.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qbitmap.h>

#include <kconfig.h>
#include <kstddirs.h>
#include <kdebug.h>

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kwin/workspace.h>
#include <kwin/options.h>
#include <kpixmapeffect.h>
#include <kdrawutil.h>

#include "qnix_client.h"

#define BUTTON_SIZE 				13
#define MENU_BUTTON_SIZE 		16
#define MINI_BUTTON_SIZE		12
#define TITLE_PIX_WIDTH			32
#define TITLE_PIX_HEIGHT		20

using namespace KWinInternal;

static void drawButtonFrame( KPixmap *pix, const QColorGroup &g, bool sunken );
static void drawTitleDetails( KPixmap *pix, bool active );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

static bool pixmaps_created = false;
static bool showMenuButtonIcon;
static bool showLeftDetail;
static KPixmap* aTitlePix = 0;
static KPixmap* iTitlePix = 0;
static KPixmap* titlePixRight = 0;
static KPixmap* aTitlePixLeft = 0;
static KPixmap* iTitlePixLeft = 0;

static QPixmap* defaultMenuPix;
static QPixmap* leftPix;
static QPixmap* iLeftPix;

static KPixmap *btnPix1;
static KPixmap *iBtnPix1;
static KPixmap *btnDownPix1;
static KPixmap *iBtnDownPix1;

static KPixmap *miniBtnPix1;
static KPixmap *iMiniBtnPix1;
static KPixmap *miniBtnDownPix1;
static KPixmap *iMiniBtnDownPix1;

static KPixmap* btnMenuPix;

static void create_pixmaps()
{
	if(pixmaps_created)
			return;

	pixmaps_created = true;
	QColor c(options->color(Options::ButtonBg, false));

	// active title gradient
	// -------------------------
	aTitlePix = new KPixmap();
	aTitlePix->resize(TITLE_PIX_WIDTH, TITLE_PIX_HEIGHT);
	KPixmapEffect::gradient(*aTitlePix,options->color(Options::TitleBar, true), options->color(Options::TitleBlend, true), KPixmapEffect::VerticalGradient);
 	drawTitleDetails( aTitlePix, true );

	// inactive title gradient
	// -------------------------
	iTitlePix = new KPixmap();
	iTitlePix->resize(TITLE_PIX_WIDTH, TITLE_PIX_HEIGHT);
	// reverse gradient looks better for inactive
	KPixmapEffect::gradient(*iTitlePix, options->color(Options::TitleBlend, false), options->color(Options::TitleBar, false), KPixmapEffect::VerticalGradient);
 	drawTitleDetails( iTitlePix, false );

	// right title pix
	// ------------------------
	titlePixRight = new KPixmap();
	titlePixRight->resize(TITLE_PIX_WIDTH, TITLE_PIX_HEIGHT);
	// NOTHC : KPixmapEffect::gradient(*titlePixRight, c.dark(120), c.light(110), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*titlePixRight, QColor(192, 196, 192), QColor(224, 228, 224), KPixmapEffect::VerticalGradient);

	// left title pix
	// ------------------------
	aTitlePixLeft = new KPixmap();
	aTitlePixLeft->resize(TITLE_PIX_WIDTH, TITLE_PIX_HEIGHT);
	KPixmapEffect::gradient(*aTitlePixLeft,options->color(Options::TitleBar, true).dark(115), options->color(Options::TitleBlend, true).dark(115), KPixmapEffect::VerticalGradient);
	iTitlePixLeft = new KPixmap();
	iTitlePixLeft->resize(TITLE_PIX_WIDTH, TITLE_PIX_HEIGHT);
	KPixmapEffect::gradient(*iTitlePixLeft,options->color(Options::TitleBar, false).dark(115), options->color(Options::TitleBlend, false).dark(115), KPixmapEffect::VerticalGradient);


	// Default icon
	defaultMenuPix = new QPixmap(kdelogo);
	// left detail icon
	leftPix = new QPixmap( qnixlogo );
	iLeftPix = new QPixmap( iqnixlogo );


	// Buttons
	btnPix1 = new KPixmap;
	btnDownPix1 = new KPixmap;
	iBtnPix1 = new KPixmap;
	iBtnDownPix1 = new KPixmap;
	miniBtnPix1 = new KPixmap;
	miniBtnDownPix1 = new KPixmap;
	iMiniBtnPix1 = new KPixmap;
	iMiniBtnDownPix1 = new KPixmap;

	// buttons (active/inactive, sunken/unsunken)
	QColorGroup g = options->colorGroup(Options::ButtonBg, true);
	c = g.background();
	btnPix1->resize(BUTTON_SIZE, BUTTON_SIZE);
	btnDownPix1->resize(BUTTON_SIZE, BUTTON_SIZE);
	iBtnPix1->resize(BUTTON_SIZE, BUTTON_SIZE);
	iBtnDownPix1->resize(BUTTON_SIZE, BUTTON_SIZE);

	miniBtnPix1->resize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);
	miniBtnDownPix1->resize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);
	iMiniBtnPix1->resize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);
	iMiniBtnDownPix1->resize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);

	KPixmapEffect::gradient(*btnPix1, QColor(214, 218, 214), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*btnDownPix1, QColor(174, 178, 174), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*miniBtnPix1, QColor(216, 220, 216), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*miniBtnDownPix1, QColor(174, 178, 174), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);

	g = options->colorGroup(Options::ButtonBg, false);
	c = g.background();
	KPixmapEffect::gradient(*iBtnPix1, QColor(214, 218, 214), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*iBtnDownPix1, QColor(174, 178, 174), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*iMiniBtnPix1, QColor(216, 220, 216), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	KPixmapEffect::gradient(*iMiniBtnDownPix1, QColor(174, 178, 174), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	// draw active buttons frames
	g = options->colorGroup(Options::ButtonBg, true);
	drawButtonFrame(btnPix1, g, false);
	drawButtonFrame(btnDownPix1, g, true);
	drawButtonFrame(miniBtnPix1, g, false);
	drawButtonFrame(miniBtnDownPix1, g, true);
	// draw inactive buttons frames
	g = options->colorGroup(Options::ButtonBg, false);
	drawButtonFrame(iBtnPix1, g, false);
	drawButtonFrame(iBtnDownPix1, g, true);
	drawButtonFrame(iMiniBtnPix1, g, false);
	drawButtonFrame(iMiniBtnDownPix1, g, true);

	// menu button
	btnMenuPix = new KPixmap;
	btnMenuPix->resize(BUTTON_SIZE, BUTTON_SIZE-2);
	KPixmapEffect::gradient(*btnMenuPix, QColor(214, 218, 214), QColor(184, 184, 184), KPixmapEffect::VerticalGradient);
	drawButtonFrame( btnMenuPix, g, false );

}

static void delete_pixmaps()
{
  delete aTitlePix;
  delete iTitlePix;
  delete titlePixRight;
	delete aTitlePixLeft;
	delete iTitlePixLeft;

	delete btnPix1;
  delete iBtnPix1;
  delete btnDownPix1;
  delete iBtnDownPix1;

  delete miniBtnPix1;
  delete iMiniBtnPix1;
  delete miniBtnDownPix1;
  delete iMiniBtnDownPix1;

	delete defaultMenuPix;
	delete leftPix;
	delete iLeftPix;
	delete btnMenuPix;
}

static void drawTitleDetails( KPixmap *pix, bool active )
{
  QPainter p;
  int x2 = pix->width();
  int y2 = pix->height();
  p.begin(pix);

	// draw details
	// top of titlebar - v dark
	p.setPen( options->color(Options::TitleBlend, active ).dark( 160 )  );
	p.drawLine(0, 0, x2, 0);
	// top of titlebar - light
	p.setPen( options->color(Options::TitleBar,active ).light( 111 ) );
	p.drawLine(0, 1, x2, 1);
	// top of titlebar - mid
	p.setPen( options->color(Options::TitleBar, active ));
	p.drawLine(0, 2, x2, 2);
	// top of titlebar - dark
	p.setPen( options->color(Options::TitleBlend, active ).dark( 130 )  );
	p.drawLine(0, 3, x2, 3);

	// bottom of titlebar - quite light
	p.setPen( options->color(Options::TitleBlend, active ).light( 110 )  );
	p.drawLine(0, y2-2, x2, y2-2);
	// bottom of titlebar - almost v dark
	// NOT HC : p.setPen( options->color(Options::TitleBlend, active ).dark( 145 )  );
	p.setPen( QColor( 151, 151, 151 ) );
	p.drawLine(0, y2-1, x2, y2-1);
	// NOT HC : p.setPen( options->color(Options::TitleBlend, active ).dark( 130 )  );
	p.setPen( QColor( 193, 197, 193 ) );
	p.drawLine(0, y2, x2 + 1, y2);
	// bottom of titlebar - v dark - only if inactive
	if( !active )
	{
		p.setPen( QColor( 68, 67, 63 ) );
		p.drawLine(1, y2, x2 - 6, y2 );
	}

}

static void drawButtonFrame( KPixmap *pix, const QColorGroup &g, bool sunken )
{
    QPainter p;
		int y1 = 0;

    int x2 = pix->width() - 1;
    int y2 = pix->height() - 1;
    p.begin(pix);

		// Outer Frame
		p.setPen( sunken ? QColor( 14, 14, 14 ) : QColor( 56, 60, 56 ) );
		// across top
		p.drawLine(0, y1, x2, y1);
		// down right
		p.drawLine(x2, y1, x2, y2);
		if( sunken)
			p.setPen( QColor( 56, 60, 56 ) );
		// across bottom
		p.drawLine(0, y2, x2, y2);
		// down left
		p.drawLine(	0, y1, 0, y2);

		// Inner Frame
		p.setPen( sunken ? QColor( 101, 101, 101) : QColor(232, 232, 232 ) );
		// top
		p.drawLine(1, y1+1, x2-2, y1+1);
		// left
		p.drawLine(	1, y1+1, 1, y2-2);
		// top right corner
		p.setPen( sunken ? QColor(91, 89, 91) : QColor(208, 204, 208 ) );
		p.drawPoint( x2-1, y1+1 );
		// bottom right corner
		p.drawPoint( 1, y2-1 );
		// bottom
		p.setPen( QColor(144, 144, 144 ) );
		p.drawLine(2, y2-1, x2-2, y2-1);
		// right
		p.setPen( sunken ? QColor( 152, 152, 152) : QColor(160, 160, 160 ) );
		p.drawLine(x2-1, 2, x2-1, y2-1);
}

static bool read_config()
{
    KConfig config("kwinqnixrc");
    config.setGroup("General");
    showMenuButtonIcon = config.readBoolEntry( "ShowMenuButtonIcon", false );
    showLeftDetail = config.readBoolEntry( "ShowLeftDetail", true );
    return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : QNiXClient
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


QNiXClient::QNiXClient(Workspace * ws, WId w, QWidget * parent, const char * name)
  : Client(ws, w, parent, name, WResizeNoErase), titlebar(0)
{

  setBackgroundMode(NoBackground);

  QFontMetrics fm(options->font(isActive(), tool_));

	// toolwindows
  if ( isTool() )
	{
  	titleHeight = 14;
    smallButtons = true;
	}
	else
	{
  	titleHeight = 22;
    smallButtons = false;
 	}

  setupLayout();

	// Make sure that the menu button uses the correct mini-icon
  iconChange();

  connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));
}

QNiXClient::~QNiXClient()
{
  // Empty.
}


void QNiXClient::setupLayout()
{
  const uint sideMargin    = 6;
  const uint bottomMargin  = 6;

	// create titlebar
	titlebar = new QSpacerItem( 0, titleHeight,  QSizePolicy::Expanding, QSizePolicy::Minimum);

	connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));

	// setup layout
	mainLayout = new QVBoxLayout(this);
	titleLayout = new QHBoxLayout();
	windowLayout = new QHBoxLayout();

	mainLayout->addLayout(titleLayout);
	mainLayout->addLayout(windowLayout, 1);
	// add bottom margin
	mainLayout->addSpacing( bottomMargin );

	// add left margin
	windowLayout->addSpacing( sideMargin );
	// add window
	windowLayout->addWidget( windowWrapper() );
	// add right margin
	windowLayout->addSpacing( sideMargin );

	// create buttons
	button[BtnMenu]    = new QNixButton(this, "menu", BtnMenu, NULL, showMenuButtonIcon, smallButtons,  i18n("Menu"));
	button[BtnClose]   = new QNixButton(this, "close", BtnClose, NULL, false, smallButtons,  i18n("Close"));
	button[BtnMin] 		 = new QNixButton(this, "iconify", BtnMin, NULL, false, smallButtons, i18n("Minimize"));
	button[BtnMax]     = new QNixButton(this, "maximize", BtnMax, NULL, false, smallButtons,  i18n("Maximize"));
	// connect buttons to their appropriate actions
	connect( button[BtnMenu],    SIGNAL(pressed()), this, SLOT( slotMenu() ));
	connect( button[BtnClose], SIGNAL(clicked()), this, SLOT( closeWindow() ) );
	connect( button[BtnMin], SIGNAL(clicked()), this, SLOT( iconify() ) );
	connect( button[BtnMax], SIGNAL(clicked()), this, SLOT( slotMaximize() ) );

	// Add everything to the main layout
	titleLayout->addSpacing(5);
	titleLayout->addWidget( button[BtnMenu] );
	titleLayout->addItem(titlebar);
	titleLayout->addSpacing(5);
	titleLayout->addWidget( button[BtnMin] );
	titleLayout->addSpacing(4);
	titleLayout->addWidget( button[BtnMax] );
	titleLayout->addSpacing(4);
	titleLayout->addWidget( button[BtnClose] );
	titleLayout->addSpacing(6);
}

void QNiXClient::paintEvent(QPaintEvent* pe)
{
	QColorGroup g;
  QPainter p(this);
  int w = width();
  int h = height();
	QRect t = titlebar->geometry();
	t.setTop(1);

	// draw titlebar right gradient (silver)
	p.drawTiledPixmap(t.width() + t.x() - 4, t.y(), width() - (t.width() + t.x() -  2) , t.height()-2, *titlePixRight );

  // outer frame ( margin )
	// -------------------------------------
	// left margin
	p.setPen(Qt::white);
	p.drawLine( 2, 19, 2, h-4);
	p.setPen( QColor( 219, 219, 219 ) );
	p.drawLine( 3, 19, 3, h-4);
	p.setPen( QColor(158, 158, 158 ) );
	p.drawLine( 4, 19, 4, h-4);
	p.setPen( QColor( 71, 71, 71 ) );
	p.drawLine( 5, 19, 5, h-4);   // innermost

	// bottom margin
	p.setPen( QColor( 153, 153, 153 ) );
	p.drawLine( 2, h-3, w-2, h-3);
	p.setPen( QColor( 219, 219, 219 ) );
	p.drawLine( 2, h-4, w-2, h-4);
	p.setPen(Qt::white);
	p.drawLine( 2, h-5, w-2, h-5);
	p.setPen( QColor( 71, 71, 71 ) );
	p.drawLine( 5, h-6, w-5, h-6);		// innermost

	// right margin
	p.setPen( QColor(158, 158, 158 ) );
	p.drawLine( w-3, 3, w-3, h-4);
	p.setPen( QColor( 219, 219, 219 ) );
	p.drawLine( w-4, 3, w-4, h-4);
	p.setPen(Qt::white);
	p.drawLine( w-5, 21, w-5, h-4);
	p.setPen( QColor( 71, 71, 71 ) );
	p.drawLine( w-6, 22, w-6, h-6);   // innermost

	// draw bottom horiz lines on right - bottom
	p.setPen( QColor( 68, 67, 63 ) );
	p.drawLine( t.width() + t.x() - 4, t.bottom(), width()-6, t.bottom() );
	p.setPen( options->color(Options::TitleBlend, false ).dark( 145 )  );
	p.drawLine( t.width() + t.x() - 4, t.bottom()-1, width()-6, t.bottom()-1 );

	// draw bottom horiz lines on right - top
	p.setPen( Qt::white );
	p.drawLine( t.width() + t.x() - 4, 2, width()-4, 2);

	// draw titlebar gradient
	// note : it must go all the way from x = 1 to cover the menu button background
	p.drawTiledPixmap( 1, t.y(), t.width()+t.x()-2, t.height()+1, isActive() ? *aTitlePix : *iTitlePix);

	// Is the client configured to show the extra detail on the left?
	if( showLeftDetail )
	{
		int lw = 20; // left detail width
		p.setPen( options->color(Options::TitleBlend, isActive() ).light( 110 )  );
		p.drawLine(2, t.y(), 2, t.height()+1);
		p.drawTiledPixmap( 2, t.y(), lw-1, t.height()+1, isActive() ? *aTitlePixLeft : *iTitlePixLeft);
		// draw split detail - 1st in titlebar colour
		p.setPen( options->color(Options::TitleBlend, isActive() ).dark( 145 )  );
		p.drawLine(lw+1, 2, lw+1, 4);
		p.drawLine(lw+2, 4, lw+2, t.bottom()-3);
		p.drawLine(lw+1, t.bottom()-3, lw+1, t.bottom()-1);
		// 2nd in dark gray
		p.setPen( QColor( 72, 72, 72 ) );
		p.drawLine(lw, 2, lw, 4);
		p.drawLine(lw+1, 4, lw+1, t.bottom()-3);
		p.drawLine(lw, t.bottom()-3, lw, t.bottom()-1);
	}

	// just an extra detail on left top corner
	p.setPen( options->color(Options::TitleBar,isActive() ).light( 110 ) );
	p.drawLine (1, 4, 1, t.bottom() - 2);
	p.setPen( options->color(Options::TitleBar,isActive() ).dark( 110 ) );
	p.drawLine (1, t.bottom() - 2, 1, t.bottom());

	// draw bottom horiz lines
	p.setPen( options->color(Options::TitleBlend, isActive() ).dark( 145 )  );
	p.drawLine(1, t.bottom()-1, t.right(), t.bottom()-1);

	// draw split detail - 1st in titlebar colour
	p.drawLine(t.width()+t.x()-1, 2, t.width()+t.x()-1, 4);
	p.drawLine(t.width()+t.x()-2, 4, t.width()+t.x()-2, t.bottom()-3);
	p.drawLine(t.width()+t.x()-1, t.bottom()-3, t.width()+t.x()-1, t.bottom()-1);
	// 2nd in dark gray
	p.setPen( QColor( 72, 72, 72 ) );
	p.drawLine(t.width()+t.x(), 2, t.width()+t.x(), 4);
	p.drawLine(t.width()+t.x()-1, 4, t.width()+t.x()-1, t.bottom()-3);
	p.drawLine(t.width()+t.x(), t.bottom()-3, t.width()+t.x(), t.bottom()-1);
	// 3rd in light gray
	// NOT HC : p.setPen( options->color(Options::TitleBlend, isActive() ).dark( 130 )  );
	p.setPen( QColor( 193, 197, 193 ) );
	p.drawLine(t.width()+t.x()+1, 2, t.width()+t.x()+1, 4);
	p.drawLine(t.width()+t.x(), 4, t.width()+t.x(), t.bottom()-3);
	p.drawLine(t.width()+t.x()+1, t.bottom()-3, t.width()+t.x()+1, t.bottom()-2);

	// draw bottom line
	p.setPen( QColor( 72, 72, 72 ) );
	p.drawLine( 1, t.bottom(), width()-6, t.bottom() );

	// draw title text
	// -----------------------
	// Align text center
  //p.drawText(t, AlignCenter, caption());
 	QFont fnt = options->font(true);
	// Set smaller font size if tool window
  if ( smallButtons )
  {
       fnt.setPointSize( fnt.pointSize() - 3 );  // Shrink font by 3pt
       fnt.setWeight( QFont::Normal );           // and disable bold
  }
	p.setFont( fnt );
  p.setPen( options->color(Options::Font, isActive() ));
  p.drawText( t.x() + 14, 1, t.width()-13, t.height()-1, AlignLeft | AlignVCenter, caption() );

	// Outer Frames
	// -----------------------------
	// outer border around whole window
	p.setPen(Qt::black);
 	p.drawRect(0, 0, w, h);
	// inner border around whole window
	p.setPen( QColor( 72, 72, 72 ) );
 	p.drawRect(1, 1, w-2, h-2);
}

void QNiXClient::resizeEvent(QResizeEvent * e)
{
  Client::resizeEvent(e);
  repaint();
}

void QNiXClient::iconChange()
{
	// should we show the menu icon?
	if( showMenuButtonIcon )
	{
		if(!miniIcon().isNull())
			button[BtnMenu]->setPixmap(miniIcon());
		else
			button[BtnMenu]->setPixmap(*defaultMenuPix);

		if (button[BtnMenu]->isVisible())
			button[BtnMenu]->repaint(false);
	}
	// special default menu button when left detail is on
	else if ( showLeftDetail )
	{
		if( isActive() )
			button[BtnMenu]->setPixmap( *leftPix );
		else
			button[BtnMenu]->setPixmap( *iLeftPix );

		if (button[BtnMenu]->isVisible())
			button[BtnMenu]->repaint(false);
	}

}


void QNiXClient::captionChange(const QString &)
{
  repaint();
}

void QNiXClient::showEvent(QShowEvent *)
{
  repaint();
}

void QNiXClient::windowWrapperShowEvent(QShowEvent *)
{
  repaint();
}

void QNiXClient::mouseDoubleClickEvent(QMouseEvent * e)
{
  if (titlebar->geometry().contains(e->pos()))
    workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
}

void QNiXClient::stickyChange(bool b)
{
  emit(stkyChange(b));
}

void QNiXClient::maximizeChange(bool b)
{
  emit(maxChange(b));
}

Client::MousePosition QNiXClient::mousePosition(const QPoint & p) const
{
  int x = p.x();
  int y = p.y();

  if (y < titlebar->geometry().height())
  {
    return Client::Center;
  }
  else if (y < height() - 20)
  {
    if (x < 4)
      return Client::Left;
    else
      return Client::Right;
  }
  else
  {
    if (x < 20)
      return Client::BottomLeft;
    else
      if (x > width() - 20)
        return Client::BottomRight;
      else
        return Client::Bottom;
  }

  return Client::mousePosition(p);
}

void QNiXClient::slotReset()
{
	read_config();
  button[BtnMax]->reset();
	button[BtnClose]->reset();
	button[BtnMin]->reset();
  repaint();
}

void QNiXClient::activeChange( bool )
{
	// Update active/inactive default icon
	if( showLeftDetail )
		iconChange();

  button[BtnMax]->repaint(false);
	button[BtnClose]->repaint(false);
	button[BtnMin]->repaint(false);
	button[BtnMenu]->repaint(false);
  repaint(false);
}

void QNiXClient::slotMaximize()
{
/*
    switch ( button[BtnMaximize]->last_button ) {
    case MidButton:
       maximize( MaximizeVertical );
       break;
    case RightButton:
       maximize( MaximizeHorizontal );
       break;
    default: //LeftButton:
       maximize( MaximizeFull );
       break;
    }
		*/
		 maximize( MaximizeFull );
}

void QNiXClient::slotMenu()
{
	// from redmond.cpp
	static QTime* t = 0;
  static QNiXClient* tc = 0;
  if ( !t )
  	t = new QTime;

	if ( tc != this || t->elapsed() > QApplication::doubleClickInterval() )
  {
  	// KS - move the menu left by 3 pixels, and down 2 pixels.
    QPoint menupoint ( button[BtnMenu]->rect().bottomLeft().x()-3, button[BtnMenu]->rect().bottomLeft().y()+2 );
    workspace()->clientPopup(this)->popup( button[BtnMenu]->mapToGlobal( menupoint ));
 	}
  else
	{
  	closeWindow();
	}
  t->start();
  tc = this;
}


void QNiXClient::animateIconifyOrDeiconify(bool /* iconify */)
{
  // do nothing
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : QNixButton
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
QNixButton::QNixButton(Client *parent, const char *name, Buttons type, const unsigned char *bitmap, bool menuButton, bool isMini, const QString& tip)
    : KWinButton(parent, name, tip)
{
	// Eliminate background flicker
	setBackgroundMode( QWidget::NoBackground );

	m_type = type;
	menuBtn = menuButton;
	miniBtn = isMini;
	client  = parent;

	// Use larger button for the menu, or mini-buttons for toolwindows.
	if ( isMini )
	{
		setFixedSize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);
		resize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE);
	}
	else if ( menuButton )
	{
		setFixedSize(MENU_BUTTON_SIZE, MENU_BUTTON_SIZE);
		resize(MENU_BUTTON_SIZE, MENU_BUTTON_SIZE);
	}
	else
	{
		setFixedSize(BUTTON_SIZE, BUTTON_SIZE);
		resize(BUTTON_SIZE, BUTTON_SIZE);
	}

	if(bitmap)
		setBitmap(bitmap);
}


QSize QNixButton::sizeHint() const
{
	if ( miniBtn )
			return(QSize(MINI_BUTTON_SIZE, MINI_BUTTON_SIZE));
	else if ( menuBtn )
			return(QSize(MENU_BUTTON_SIZE, MENU_BUTTON_SIZE));
	else
			return(QSize(BUTTON_SIZE, BUTTON_SIZE));
}


void QNixButton::reset()
{
   repaint(false);
}


void QNixButton::setBitmap(const unsigned char *bitmap)
{
  pix.resize(0, 0);
  deco = QBitmap(10, 10, bitmap, true);
  deco.setMask(deco);
	repaint( false );
}


void QNixButton::setPixmap( const QPixmap &p )
{
	deco.resize(0, 0);
	pix = p;

	if (miniBtn)
		setMask(QRect(0, 0, MINI_BUTTON_SIZE, MINI_BUTTON_SIZE));
	else if (menuBtn)
		setMask(QRect(0, 0, MENU_BUTTON_SIZE, MENU_BUTTON_SIZE+1));
	else
		setMask(QRect(0, 0, BUTTON_SIZE, BUTTON_SIZE));

	repaint( false );
}


void QNixButton::mousePressEvent( QMouseEvent* e )
{
	last_button = e->button();
	QMouseEvent me ( e->type(), e->pos(), e->globalPos(),
					 LeftButton, e->state() );
	KWinButton::mousePressEvent( &me );
}


void QNixButton::mouseReleaseEvent( QMouseEvent* e )
{
	last_button = e->button();
	QMouseEvent me ( e->type(), e->pos(), e->globalPos(), LeftButton, e->state() );
	KWinButton::mouseReleaseEvent( &me );
}

void QNixButton::drawButton(QPainter *p)
{
	int w = width();
	int h = height();

	// menu button
	if( !pix.isNull() )
	{
		// draw part of titlebar that was covered up
		QPixmap px;
		px.resize( w, h+2 );
		bitBlt( &px, 0, 0, (client->isActive() ? aTitlePixLeft : iTitlePixLeft), 1, 2, w, h+2 );
 		p->drawPixmap( 0, 0, px );
		// draw image
		if ( menuBtn && miniBtn )
    {
    	QPixmap tmpPix;
      // Smooth scale the menu button pixmap
      tmpPix.convertFromImage( pix.convertToImage().smoothScale(12, 12));
      p->drawPixmap( 0, 0, tmpPix );
	  }
		else
    	p->drawPixmap( 0, 0, pix );
		// finished
		return;
	}

	// Never paint if the pixmaps have not been created
	if( !pixmaps_created )
		return;

	// special menu button - detail without pixmap
	if( m_type == BtnMenu )
	{
		p->drawPixmap(0, 2, *btnMenuPix );
		// draw part of titlebar that was covered up
		p->setPen( options->color(Options::TitleBlend, client->isActive() ).dark( 130 )  );
		p->drawLine(0, 0, BUTTON_SIZE, 0);
		p->setPen( options->color(Options::TitleBar, client->isActive()) );
		p->drawLine(0, 1, BUTTON_SIZE, 1);
		// pattern
		p->setPen( QColor(144, 144, 144 ) );
		p->drawLine( 1, 5, w - 2, 5 );  // 1.1
		p->drawLine( 1, 8, w - 2, 8 );  // 2.1
		//p->setPen( QColor(56, 60, 56 ) );
		p->setPen( QColor(176, 172, 176) );
		p->drawLine( 1, 6, w - 2, 6);  // 1.2
		p->drawLine( 1, 9, w - 2, 9 );  // 2.1

		return;
	}

	// Draw background pixmap
	if(client->isActive())
	{
		if(isDown())
			p->drawPixmap(0, 0, miniBtn ? *miniBtnDownPix1 : *btnDownPix1);
		else
			p->drawPixmap(0, 0, miniBtn ? *miniBtnPix1 : *btnPix1);
	}
	else
	{
		if(isDown())
			p->drawPixmap(0, 0, miniBtn ? *iMiniBtnDownPix1 : *iBtnDownPix1);
		else
			p->drawPixmap(0, 0, miniBtn ? *iMiniBtnPix1 : *iBtnPix1);
	}

	// Draw specific detail
	switch( m_type )
	{
		case( BtnMax ):
			if( isDown() )
			{
				// extra line
				p->setPen( QColor(162, 162, 162) );
				p->drawLine( 2, 2, 2, h - 2 );
				// the rest
				p->setPen( QColor(176, 172, 176) );
				p->drawLine( 2, 4, w - 3, 4 );
				p->setPen( QColor(200, 200, 200) );
				p->drawLine( 2, 5, w - 3, 5 );
				p->setPen( QColor(198, 198, 198) );
				p->drawPoint( w-2, 5 );

			}
			else
			{
				p->setPen( QColor(176, 172, 176) );
				p->drawLine( 1, 4, w - 3, 4 );
				p->setPen( QColor(229, 229, 229) );
				p->drawLine( 1, 5, w - 3, 5 );
				p->setPen( QColor(216, 212, 216) );
				p->drawPoint( w-2, 5 );
			}
			break;

		case( BtnClose ):
			// NOTE : same for up/down
			// outer rectangle
			p->setPen( QColor(144, 144, 144 ) );
			p->drawLine( 3, 3, 3, 8 );  // down left
			p->drawLine( 3, 3, 9, 3 );  // across top
			p->setPen( QColor(232, 232, 232 ) );
			p->drawLine( 9, 3, 9, 8 ); // down right
			p->drawLine( 3, 9, 9, 9 );  // across bottom
			p->setPen( QColor(192, 188, 192 ) );
			p->drawPoint( 9, 3 ); // right top corner
			p->drawPoint( 3, 9 ); // left bottom corner
			// inner rectangle
			p->setPen( QColor(56, 60, 56 ) );
			p->drawRect( 4, 4, 5, 5 );
			p->setPen( QColor(192, 188, 192 ) );
			p->drawRect( 5, 5, 2, 2 );
			p->setPen( QColor(128, 128, 128 ) );
			p->drawRect( 6, 6, 2, 2 );
			p->setPen( QColor(160, 160, 160 ) );
			p->drawLine( 5, 7, 7, 5 );
			break;

		case( BtnMin ):
			if( isDown() )
			{
				// extra line
				p->setPen( QColor(162, 162, 162) );
				p->drawLine( 2, 2, 2, h - 2 );
				// the rest
				p->setPen( QColor(176, 172, 176) );
				p->drawLine( 2, 4, w - 3, 4 );
				p->setPen( QColor(200, 200, 200) );
				p->drawLine( 2, 5, w - 3, 5 );
				p->setPen( QColor(198, 198, 198) );
				p->drawPoint( w-2, 5 );
			}
			else
			{
				p->setPen( QColor(208, 204, 208 ) );
				p->drawPoint( 1, h-6 );
			}
			// the rest is common for up/down
			p->setPen( QColor(144, 144, 144 ) );
			p->drawLine( 2, h - 6, w - 2, h - 6 );  // horiz line 1
			p->setPen( QColor(56, 60, 56 ) );
			p->drawLine( 1, h - 5, w - 2, h - 5 );  // horiz line 2
			// arrow:
			p->setPen( QColor(80, 80, 80) );
			p->drawLine( 4, 3, 8, 3 );
			p->drawLine( 5, 4, 7, 4 );
			p->drawPoint( 6, 5 );
			break;

		default:
			break;

	}

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INIT
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern "C"
{
  Client * allocate(Workspace * ws, WId w, int tool)
  {
    return new QNiXClient( ws, w );
  }
	void init()
	{
		create_pixmaps();
    read_config();
	}
	void reset()
	{
		delete_pixmaps();
		create_pixmaps();
     read_config();
		// Ensure change in tooltip state gets applied
		Workspace::self()->slotResetAllClientsDelayed();
	}
	void deinit()
	{
		delete_pixmaps();
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "qnix_client.moc"
